@@ -22,7 +22,7 @@ from pydantic_ai.messages import (
     ToolReturn,
     ToolReturnPart,
 )
-from pydantic_ai.models.openai import OpenAIModelSettings
+from pydantic_ai.models.openai import OpenAIChatModelSettings
 from pydantic_ai.settings import ModelSettings
 
 # Consolidated relative imports
@@ -31,9 +31,11 @@ from code_puppy.config import (
     get_compaction_strategy,
     get_compaction_threshold,
     get_global_model_name,
+    get_openai_reasoning_effort,
     get_protected_token_count,
     get_value,
-    load_mcp_server_configs, get_message_limit,
+    load_mcp_server_configs,
+    get_message_limit,
 )
 from code_puppy.mcp_ import ServerConfig, get_mcp_manager
 from code_puppy.messaging import (
@@ -829,9 +831,11 @@ class BaseAgent(ABC):
 
         model_settings: ModelSettings = ModelSettings(**model_settings_dict)
         if "gpt-5" in model_name:
-            model_settings_dict["openai_reasoning_effort"] = "off"
+            model_settings_dict["openai_reasoning_effort"] = (
+                get_openai_reasoning_effort()
+            )
             model_settings_dict["extra_body"] = {"verbosity": "low"}
-            model_settings = OpenAIModelSettings(**model_settings_dict)
+            model_settings = OpenAIChatModelSettings(**model_settings_dict)
 
         self.cur_model = model
         p_agent = PydanticAgent(
