@@ -3,12 +3,13 @@
 from typing import Optional
 
 import camoufox
-from playwright.async_api import Browser, BrowserContext, Page, Playwright, async_playwright
+from camoufox.addons import DefaultAddons
+from camoufox.exceptions import CamoufoxNotInstalled, UnsupportedVersion
+from camoufox.locale import ALLOW_GEOIP, download_mmdb
+from camoufox.pkgman import CamoufoxFetcher, camoufox_path
+from playwright.async_api import Browser, BrowserContext, Page
 
 from code_puppy.messaging import emit_info
-from camoufox.pkgman import CamoufoxFetcher
-from camoufox.locale import ALLOW_GEOIP, download_mmdb
-from camoufox.addons import maybe_download_addons, DefaultAddons
 
 
 class CamoufoxManager:
@@ -17,7 +18,6 @@ class CamoufoxManager:
     _instance: Optional["CamoufoxManager"] = None
     _browser: Optional[Browser] = None
     _context: Optional[BrowserContext] = None
-    _playwright: Optional[Playwright] = None
     _initialized: bool = False
 
     def __new__(cls):
@@ -52,27 +52,17 @@ class CamoufoxManager:
 
         try:
             emit_info("[yellow]Initializing Camoufox (privacy Firefox)...[/yellow]")
-            
+
             # Ensure Camoufox binary and dependencies are fetched before launching
             await self._prefetch_camoufox()
 
-            try:
-                await self._initialize_camoufox()
-                emit_info(
-                    "[green]✅ Camoufox initialized successfully (privacy-focused Firefox)[/green]"
-                )
-            except Exception as camoufox_error:
-                error_reason = str(camoufox_error).splitlines()[0]
-                emit_info(
-                    "[yellow]⚠️ Camoufox failed to initialize, falling back to Playwright Firefox[/yellow]"
-                )
-                await self._cleanup()
-                await self._initialize_playwright_firefox(error_reason)
-
+            await self._initialize_camoufox()
+            emit_info(
+                "[green]✅ Camoufox initialized successfully (privacy-focused Firefox)[/green]"
+            )
             self._initialized = True
 
-        except Exception as e:
-            emit_info(f"[red]❌ Failed to initialize browser: {e}[/red]")
+        except Exception:
             await self._cleanup()
             raise
 
@@ -82,6 +72,8 @@ class CamoufoxManager:
             headless=self.headless,
             block_webrtc=self.block_webrtc,
             humanize=self.humanize,
+            exclude_addons=list(DefaultAddons),
+            addons=[],
         )
         self._browser = await camoufox_instance.start()
         self._context = await self._browser.new_context(
@@ -91,20 +83,6 @@ class CamoufoxManager:
         page = await self._context.new_page()
         await page.goto(self.homepage)
 
-    async def _initialize_playwright_firefox(self, error_reason: str) -> None:
-        """Fallback to vanilla Playwright Firefox when Camoufox fails."""
-        self._playwright = await async_playwright().start()
-        self._browser = await self._playwright.firefox.launch(headless=self.headless)
-        self._context = await self._browser.new_context(
-            viewport={"width": 1920, "height": 1080},
-            ignore_https_errors=True,
-        )
-        page = await self._context.new_page()
-        await page.goto(self.homepage)
-        emit_info(
-            f"[green]✅ Playwright Firefox fallback ready (Camoufox error: {error_reason})[/green]"
-        )
-
     async def get_current_page(self) -> Optional[Page]:
         """Get the currently active page."""
         if not self._initialized or not self._context:
@@ -127,18 +105,28 @@ class CamoufoxManager:
 
     async def _prefetch_camoufox(self) -> None:
         """Prefetch Camoufox binary and dependencies."""
-        emit_info("[cyan]🔍 Ensuring Camoufox binary and dependencies are up-to-date...[/cyan]")
-        
-        # Fetch Camoufox binary if needed
-        CamoufoxFetcher().install()
-        
+        emit_info(
+            "[cyan]🔍 Ensuring Camoufox binary and dependencies are up-to-date...[/cyan]"
+        )
+
+        needs_install = False
+        try:
+            camoufox_path(download_if_missing=False)
+            emit_info("[cyan]🗃️ Using cached Camoufox installation[/cyan]")
+        except (CamoufoxNotInstalled, FileNotFoundError):
+            emit_info("[cyan]📥 Camoufox not found, installing fresh copy[/cyan]")
+            needs_install = True
+        except UnsupportedVersion:
+            emit_info("[cyan]♻️ Camoufox update required, reinstalling[/cyan]")
+            needs_install = True
+
+        if needs_install:
+            CamoufoxFetcher().install()
+
         # Fetch GeoIP database if enabled
         if ALLOW_GEOIP:
             download_mmdb()
-        
-        # Download default addons
-        maybe_download_addons(list(DefaultAddons))
-        
+
         emit_info("[cyan]📦 Camoufox dependencies ready[/cyan]")
 
     async def close_page(self, page: Page) -> None:
@@ -160,9 +148,6 @@ class CamoufoxManager:
             if self._browser:
                 await self._browser.close()
                 self._browser = None
-            if self._playwright:
-                await self._playwright.stop()
-                self._playwright = None
             self._initialized = False
         except Exception as e:
             emit_info(f"[yellow]Warning during cleanup: {e}[/yellow]")
@@ -184,7 +169,7 @@ class CamoufoxManager:
                     loop.create_task(self._cleanup())
                 else:
                     loop.run_until_complete(self._cleanup())
-            except:
+            except Exception:
                 pass  # Best effort cleanup
 
 
